from time import sleep
from xmltodict import parse
from modutils import BaseSession
from logging import CRITICAL, getLogger

class GoNews(BaseSession):
    states = {
        'Alabama': 'AL',
        'Alaska': 'AK',
        'Arizona': 'AZ',
        'Arkansas': 'AR',
        'California': 'CA',
        'Colorado': 'CO',
        'Connecticut': 'CT',
        'Delaware': 'DE',
        'District of Columbia': 'DC',
        'Florida': 'FL',
        'Georgia': 'GA',
        'Hawaii': 'HI',
        'Idaho': 'ID',
        'Illinois': 'IL',
        'Indiana': 'IN',
        'Iowa': 'IA',
        'Kansas': 'KS',
        'Kentucky': 'KY',
        'Louisiana': 'LA',
        'Maine': 'ME',
        'Maryland': 'MD',
        'Massachusetts': 'MA',
        'Michigan': 'MI',
        'Minnesota': 'MN',
        'Mississippi': 'MS',
        'Missouri': 'MO',
        'Montana': 'MT',
        'Nebraska': 'NE',
        'Nevada': 'NV',
        'New Hampshire': 'NH',
        'New Jersey': 'NJ',
        'New Mexico': 'NM',
        'New York': 'NY',
        'North Carolina': 'NC',
        'North Dakota': 'ND',
        'Ohio': 'OH',
        'Oklahoma': 'OK',
        'Oregon': 'OR',
        'Pennsylvania': 'PA',
        'Rhode Island': 'RI',
        'South Carolina': 'SC',
        'South Dakota': 'SD',
        'Tennessee': 'TN',
        'Texas': 'TX',
        'Utah': 'UT',
        'Vermont': 'VT',
        'Virginia': 'VA',
        'Washington': 'WA',
        'West Virginia': 'WV',
        'Wisconsin': 'WI',
        'Wyoming': 'WY'
    }

    def __init__(self):
        super().__init__()
        self.resolve_status_codes.extend([400, 404, 500])
        self.base_url = 'https://news.google.com/news/rss'
        self.session_logger.setLevel(CRITICAL)
        getLogger('urllib3').setLevel(CRITICAL)


    def search(self, query: str = None, has_words: list = None, exclude_words: list = None, when: str = '1d'):
        """search google news feed

        :param query: Exact query to search
        :param has_words: words that news articles should contain
        :param exclude_words:
        :param when:
        :return:
        """
        full_query = f'"{query}" when:{when} '
        if has_words:
            if isinstance(has_words, str):
                full_query += f'{has_words} '
            elif isinstance(has_words, list):
                full_query += f'{" ".join(has_words)} '
        if exclude_words:
            if isinstance(exclude_words, str):
                full_query += f'-{exclude_words} '
            elif isinstance(exclude_words, list):
                full_query += f'{" ".join([f"-{w}" for w in exclude_words])} '
        return self.get(f'{self.base_url}/search', params={'q': full_query})

    def search_stories(self, query: str = None, has_words: list = None, exclude_words: list = None, when: str = '1d',
                       max_stories: int = None):
        news: dict = {}
        resp = self.search(query, has_words=has_words, exclude_words=exclude_words, when=when)
        if resp.status_code == 200:
            news = parse(resp.content.decode('utf-8'))
        self.print_news(self.parse_news(news), max_stories=max_stories)

    def get_news_by_location(self, city: str, state_abbr: str):
        return self.get(f'{self.base_url}/headlines/section/geo/{city}{state_abbr}')

    def get_news_top_stories(self):
        return self.get(f'{self.base_url}/news/rss')

    def parse_news(self, news: dict):
        results: dict = {}
        for index, item in enumerate(news.get('rss', {}).get('channel', {}).get('item', [])):
            results.update({str(index+1): {'title': item.get('title', None), 'link': item.get('link', None)}})
        return results

    def top_stories_by_location(self, city: str, state: str, max_stories: int = None):
        news: dict = {}
        assert any([state.lower() == st_key.lower() for st_key in self.states.keys()]) or any(
            [state.lower() == st_abbr.lower() for st_abbr in self.states.values()]), f'{state!r} is not a valid ' \
                                                                                     f'state name or state abbreviation'
        resp = self.get_news_by_location(city, state)
        if resp.status_code == 200:
            news = parse(resp.content.decode('utf-8'))
        self.print_news(self.parse_news(news), max_stories=max_stories)

    def top_stories(self, max_stories: int = None):
        news: dict = {}
        resp = self.get_news_top_stories()
        if resp.status_code == 200:
            news = parse(resp.content.decode('utf-8'))
        self.print_news(self.parse_news(news), max_stories=max_stories)

    def resolve_redirect(self, link: str):
        resp = self.get(link)
        return resp.url

    def print_news(self, parsed_news: dict, max_stories: int = None):
        if parsed_news:
            for headline_number, content in parsed_news.items():
                if max_stories and int(headline_number) > max_stories:
                    break
                print(f'{headline_number}. {content["title"]}\nURL: {self.resolve_redirect(content["link"])}\n\n')
                sleep(1)

        else:
            print(f'No news was found')




import click

from typing import List

from gonews.utils import GoNews


news = GoNews()


@click.group('cli')
def cli():
    pass


@cli.command('top-stories')
@click.option('--max-stories', '-ms', type=int, default=None, required=False, help='Max number of stories to retrieve')
def top_stories(max_stories: int = None):
    """Print current top stories"""
    news.top_stories(max_stories=max_stories)


@cli.command('top-stories-by-location')
@click.option('--city', '-c', type=str, required=True, help='City name')
@click.option('--state', '-s',
              type=click.Choice(list(news.states.keys())+list(news.states.values()), case_sensitive=False),
              required=True, help='State name. NOTE: Not case sensitive')
@click.option('--max-stories', '-ms', type=int, default=None, required=False, help='Max number of stories to retrieve')
def top_stories_by_location(city:str, state: str, max_stories: int = None):
    """Print current top stories for city, state"""
    news.top_stories_by_location(city, state, max_stories=max_stories)


@cli.command('search-stories')
@click.option('--query', '-q', type=str, required=True, help='Exact search term')
@click.option('--has-word', '-hw', type=str, required=False, multiple=True, help='Stories should have given word')
@click.option('--exclude-word', '-ew', type=str, required=False, multiple=True,
              help='Stories should not contain given word')
@click.option('--timeframe', '--t', type=click.Choice(['1d', '7d', '14d', '30d', '1y']), required=False, default='1d',
              help='Stories from this timeframe')
@click.option('--max-stories', '-ms', type=int, default=None, required=False, help='Max number of stories to retrieve')
def search_stories(query: str, has_word: List[str], exclude_word: List[str], timeframe: str, max_stories: int = None):
    """Print top stories based on search"""
    has_word = list(has_word)
    exclude_word = list(exclude_word)
    news.search_stories(query, has_words=has_word, exclude_words=exclude_word, when=timeframe, max_stories=max_stories)


if __name__ == '__main__':
    cli()


import os

from setuptools import find_namespace_packages, setup

PATH = os.path.dirname(__file__)

with open(os.path.join(PATH, 'README.md')) as fin:
    README = fin.read()

setup(
    name='gonews',
    version='0.0.1',
    packages=find_namespace_packages(include=['gonews', 'gonews.*']),
    include_package_data=True,
    description='A simple to use CLI for viewing Google News top stories and searching for specific stories',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://www.github.com/tannerburns/gonews',
    author='Tanner Burns',
    author_email='tjburns102@gmail.com',
    install_requires=[
        'click',
        'xmltodict',
        'modutils'
    ],
    entry_points={
        "console_scripts": [
            'gonews=gonews.cli:cli'
        ]
    }
)